@@ -16,9 +16,9 @@ class Tenant:
     Mandatory parameters
     --------------------
     Either provide all of the below named parameters:
-        api_key : an api key for the tenant
-        tenant : the fqdn of the tenant, e.g. test.us.qlikcloud.com
-        tenant_id : the tenant id of the tenant, found in "Settings"
+        api_key: an api key for the tenant
+        tenant: the fqdn of the tenant, e.g. test.us.qlikcloud.com
+        tenant_id: the tenant id of the tenant, found in "Settings"
 
     OR provide a path to a config.json file that has the structure:
         {
@@ -31,11 +31,16 @@ class Tenant:
 
     Example Usage
     --------------------
-       tenant1 = qsaas.Tenant(api_key_1, tenant_1, tenant_id_1)
-       tenant2 = qsaas.Tenant(api_key_2, tenant_2, tenant_id_2)
+        Option 1:
+        q = Tenant(config="<file>.json")
 
-       tenant1.get('users') -> Returns users from tenant 1
-       tenant2.get('users') -> Returns users from tenant 2
+        Option 2:
+        q = Tenant(api_key=<key>, tenant=<tenant>, tenant_id=<id>)
+
+        Multiple tenants:
+        q_us = Tenant(config="us.json")
+        q_emea = Tenant(config="emea.json")
+        q_apac = Tenant(config="apac.json")
     """
 
     def __init__(self, api_key=False, tenant=False, tenant_id=False,
@@ -161,7 +166,7 @@ class Tenant:
         s.close()
         return result
 
-    def post(self, endpoint, body):
+    def post(self, endpoint, body, params={}):
         """
         Description
         --------------------
@@ -179,6 +184,10 @@ class Tenant:
         endpoint (str), exclude api/{version}
         body (dict), occasionally requires manual json.dumps()
 
+        Optional parameters
+        --------------------
+        params (dict)
+
         Example Usage
         --------------------
         Example:
@@ -186,9 +195,9 @@ class Tenant:
                  {"spaceId": '<SpaceId>'})
         """
 
-        return self._generic('post', endpoint, body)
+        return self._generic('post', endpoint, body, params)
 
-    def put(self, endpoint, body):
+    def put(self, endpoint, body, params={}):
         """
         Description
         --------------------
@@ -206,6 +215,10 @@ class Tenant:
         endpoint (str), exclude api/{version}
         body (dict), occasionally requires manual json.dumps()
 
+        Optional parameters
+        --------------------
+        params (dict)
+
         Example Usage
         --------------------
         Example:
@@ -213,9 +226,9 @@ class Tenant:
                 json.dumps({"ownerId": '<UserId>'}))
         """
 
-        return self._generic('put', endpoint, body)
+        return self._generic('put', endpoint, body, params)
 
-    def patch(self, endpoint, body):
+    def patch(self, endpoint, body, params={}):
         """
         Description
         --------------------
@@ -233,11 +246,14 @@ class Tenant:
         endpoint (str), exclude api/{version}
         body (dict), occasionally requires manual json.dumps()
 
+        Optional parameters
+        --------------------
+        params (dict)
         """
 
-        return self._generic('patch', endpoint, body)
+        return self._generic('patch', endpoint, body, params)
 
-    def async_app_copy(self, app_id, copies=1, chunks=10, users=False):
+    def async_app_copy(self, app_id, copies=1, chunks=10, users=[]):
         """
         Description
         --------------------
@@ -306,7 +322,7 @@ class Tenant:
             sem = asyncio.Semaphore(chunks)
 
             async with ClientSession() as session:
-                if users:
+                if len(users) > 0:
                     for user_id in users:
                         for i in range(copies):
                             task = asyncio.ensure_future(
@@ -330,7 +346,7 @@ class Tenant:
         future = asyncio.ensure_future(run(app_id, copies, chunks, users))
         loop.run_until_complete(future)
 
-    def async_delete(self, endpoint, ids=False, chunks=10):
+    def async_delete(self, endpoint, ids=[], chunks=10):
         """
         Description
         --------------------
@@ -366,7 +382,7 @@ class Tenant:
                 return await call(url, session)
 
         async def run(endpoint, ids, chunks):
-            if ids:
+            if len(ids) > 0:
                 url = self.tenant + '/api/v1/' + endpoint + '/'
                 tasks = []
 
@@ -388,8 +404,8 @@ class Tenant:
         future = asyncio.ensure_future(run(endpoint, ids, chunks))
         loop.run_until_complete(future)
 
-    def async_post(self, endpoint, payloads=False, replace_char=False,
-                   replace_ids=False, chunks=10):
+    def async_post(self, endpoint, payloads=[], replace_char='',
+                   replace_ids=[], chunks=10):
         """
         Description
         --------------------
@@ -442,8 +458,8 @@ class Tenant:
         return self._async_generic('post', endpoint, payloads, replace_char,
                                    replace_ids, chunks)
 
-    def async_put(self, endpoint, payloads=False, replace_char=False,
-                  replace_ids=False, chunks=10):
+    def async_put(self, endpoint, payloads=[], replace_char='',
+                  replace_ids=[], chunks=10):
         """
         Description
         --------------------
@@ -460,8 +476,8 @@ class Tenant:
         return self._async_generic('put', endpoint, payloads, replace_char,
                                    replace_ids, chunks)
 
-    def async_patch(self, endpoint, payloads=False, replace_char=False,
-                    replace_ids=False, chunks=10):
+    def async_patch(self, endpoint, payloads=[], replace_char='',
+                    replace_ids=[], chunks=10):
         """
         Description
         --------------------
@@ -501,10 +517,10 @@ class Tenant:
 
         async def run(method, endpoint, payloads, replace_char, replace_ids,
                       chunks):
-            if payloads:
-                if replace_char and replace_ids:
+            if len(payloads) > 0:
+                if len(replace_char) > 0 and len(replace_ids) > 0:
                     fill_urls = True
-                elif any([replace_char, replace_ids]):
+                elif any([len(replace_char) > 0, len(replace_ids) > 0]):
                     raise Exception(
                         'both replace_char and replace_ids must be present')
                 else:
@@ -518,10 +534,9 @@ class Tenant:
                     if fill_urls:
                         if len(replace_ids) == len(payloads):
                             for idx, element_id in enumerate(replace_ids):
-                                endpoint = endpoint.replace(
+                                new_endpoint = endpoint.replace(
                                     replace_char, element_id)
-                                url = self.tenant + '/api/v1/' + endpoint
-
+                                url = self.tenant + '/api/v1/' + new_endpoint
                                 task = asyncio.ensure_future(
                                     bound_call(sem, method, url, session,
                                                payloads[idx]))
@@ -552,7 +567,7 @@ class Tenant:
                                            replace_char, replace_ids, chunks))
         loop.run_until_complete(future)
 
-    def _generic_request(self, s, method, endpoint, body, json=False):
+    def _generic_request(self, s, method, endpoint, body, params, json=False):
         """
         Description
         --------------------
@@ -560,13 +575,15 @@ class Tenant:
         """
         func = 's.' + method
         if not json:
-            r = eval(func)(self.tenant + '/api/v1/' + endpoint, data=body)
+            r = eval(func)(self.tenant + '/api/v1/' + endpoint,
+                           params=params, data=body)
         else:
-            r = eval(func)(self.tenant + '/api/v1/' + endpoint, json=body)
+            r = eval(func)(self.tenant + '/api/v1/' +
+                           endpoint, params=params, json=body)
 
         return r
 
-    def _generic(self, method, endpoint, body):
+    def _generic(self, method, endpoint, body, params):
         """
         Description
         --------------------
@@ -578,19 +595,21 @@ class Tenant:
         s = requests.Session()
         s.headers.update(self.auth_header)
 
-        r = self._generic_request(s, method, endpoint, body)
+        r = self._generic_request(s, method, endpoint, body, params)
 
         if r.status_code == 400:
             flag_400 = True
             body = [body]
             s.headers.update({'Content-Type': 'application/json',
                               'Accept': 'application/json'})
-            r = self._generic_request(s, method, endpoint, body, json=True)
+            r = self._generic_request(
+                s, method, endpoint, body, params, json=True)
 
             if r.status_code == 500:
                 flag_500 = True
                 body = json.dumps(body[0])
-                r = self._generic_request(s, method, endpoint, body)
+                r = self._generic_request(s, method, endpoint, body,
+                                          params)
 
             elif r.status_code == 400:
                 raise Exception(r.status_code, r.text)
@@ -598,7 +617,8 @@ class Tenant:
         if r.status_code == 500:
             flag_500 = True
             body = json.dumps(body)
-            r = self._generic_request(s, method, endpoint, body)
+            r = self._generic_request(s, method, endpoint, body,
+                                      params)
 
         if r.status_code >= 200 and r.status_code < 300:
             try:
